// codec.c
// Michael G. Morrow, 2001-2002
// codec support routines for talk-thru application

#include <c6x.h>
#include "hpi.h"
#include "..\support.h"  
#include "..\winDSK6_def.h"


/* global data */                 
#define LEFT  0
#define RIGHT 1

/* Data is received from the PCM3006 codec as 2 16-bit words (left/right) */
/* packed into one 32-bit word.  The union allows the data to be accessed */
/* as a single entity when transferring to and from the serial port, but */
/* still be able to manipulate the left and right channels independently. */

volatile union {
	unsigned int uint;
	short channel[2];
	} PcmDataIn, PcmDataOut = {(unsigned int)0};
                
interrupt void McBSP_Rx_ISR()
{                              
	McBSP *port = McBSP_Base_ToUse;

	PcmDataIn.uint = port->drr; // get input data from serial port
}

interrupt void McBSP_Tx_ISR()
{
	McBSP *port = McBSP_Base_ToUse;
	
	// implement gain
	PcmDataOut.channel[ LEFT] = PcmDataIn.channel[ LEFT] * HPI_Block.Gain;
	PcmDataOut.channel[RIGHT] = PcmDataIn.channel[RIGHT] * HPI_Block.Gain;

	port->dxr = PcmDataOut.uint; // send output data to serial port
}

interrupt void McBSP0_Rx_ISR()
{                              
	McBSP *port  = McBSP0_Base; 

	PcmDataIn.uint = port->drr; // get input data from serial port
}

interrupt void McBSP0_Tx_ISR()
{
	McBSP *port  = McBSP0_Base;
	
	// implement gain
	PcmDataOut.channel[ LEFT] = PcmDataIn.channel[ LEFT] * HPI_Block.Gain;
	PcmDataOut.channel[RIGHT] = PcmDataIn.channel[RIGHT] * HPI_Block.Gain;

	port->dxr = PcmDataOut.uint & 0xfffffffe; // send output data to serial port 
}


