
// support.c

#include <c6x.h>
#include "c6x11dsk.h"
#include "support.h"  
#include "winDSK6_def.h"
#include "hpi_min.h"

McBSP* McBSP_Base_ToUse = McBSP1_Base;

int HandleHostCodecFlags(HPI_Memory_Block* hpi)
{ // return 1 if any changes made
  		switch(hpi->CODEC) {
  		case eDSP_PCM3006: // all PCM300X codec daughtercards
  		case eDAC_PCM3003:
  		case eDAC_Audio:
  		case TI_PCM3003_16bit:
			if(hpi->TimerFlag == TIMER1_CLOCK_REQ) {
			    hpi->SampleFreq = SetNewFs_PCM300X(hpi->CODEC, hpi->TimerDivider);
			    hpi->TimerFlag = TIMER1_CLOCK_SET; 
				return 1;			
			}
			break;
  		case TLC320AD535:
	  		if(hpi->SecCommFlag == SecCommWrite) { 
  				RequestSecCommWrite(hpi->SecCommData);
  				hpi->SecCommFlag = 0;
				return 1;
	  		}
  			else if(hpi->SecCommFlag == SecCommRead) { 
  				hpi->SecCommData = RequestSecCommRead(hpi->SecCommData);
  				hpi->SecCommFlag = 0;
  				return 1;
  			}  			
			break;
  		case THS1206:
			if(hpi->TimerFlag == TIMER0_CLOCK_REQ) {
			    hpi->SampleFreq = SetTimer0Period(hpi->TimerDivider);
				hpi->TimerFlag = TIMER0_CLOCK_SET; 
				return 1;
			}
			break;
		case AIC23_16bit_McBSP01: // 6713 dsk onboard codec
		case AIC23_16bit_McBSP12: // 6416T dsk onboard codec
	  		if(hpi->SecCommFlag == SecCommWrite) { 
  				if(hpi->CODEC == AIC23_16bit_McBSP01) // 6713
  					Write_AIC23_Reg(hpi->SecCommData, McBSP0_Base);
  				else // 6416T
  					Write_AIC23_Reg(hpi->SecCommData, McBSP1_Base);
				if((hpi->SecCommData & (8<<9)) == (8<<9)) { // reprogramming sample rate
					switch(hpi->SecCommData & 0x03F) {
					case 0x01D:
						hpi->SampleFreq = 96000;
						break;
					case 0x03F:
						hpi->SampleFreq = 88235;
						break;
					case 0x023:
						hpi->SampleFreq = 44117;
						break;
					case 0x019:
						hpi->SampleFreq = 32000;
						break;
					case 0x00D:
						hpi->SampleFreq = 8000;
						break;
					default:
						hpi->SampleFreq = 48000;
						break;
					}
				}
 				hpi->SecCommFlag = 0;
				return 1;
			}
			break;
  		}
	return 0;  		
}

int Write_AIC23_Reg(unsigned short reg_data, volatile McBSP *port)
{
	volatile int i;
	
	if((reg_data & (8<<9)) == (8<<9)) { // reprogramming sample rate
		while(!(port->spcr & 0x00020000)) // wait for codec ready
			;
		port->dxr = (9<<9) | 0; // turn off AIC23 digital interface
	}

	for(i=0;i < 100000;i++)
		;

	while(!(port->spcr & 0x00020000)) // wait for codec ready
		;
	port->dxr = reg_data; // write codec register

	for(i=0;i < 100000;i++)
		;

	if((reg_data & (8<<9)) == (8<<9)) { // reprogramming sample rate
		while(!(port->spcr & 0x00020000)) // wait for codec ready
			;
		port->dxr = (9<<9) | 1; // turn on AIC23 digital interface
	}
	
	return 1; // return value ignored in this version
}

float Init_AIC23(volatile McBSP *control_port, McBSP *data_port)
///////////////////////////////////////////////////////////////////////
// Purpose:   Performs initialization of the 6713 DSK's onboard codec
//
// Input:     None
//
// Returns:   Sample frequency (float)
//
// Calls:     Nothing
//
// Notes:     None
///////////////////////////////////////////////////////////////////////
{	
	#define NUM_AIC23_REGISTERS 10
	volatile int i;
	unsigned short AIC23_data[NUM_AIC23_REGISTERS] = {
        	(0<<9) | 0x017, // reg 0 - left line volume (0 db)
        	(1<<9) | 0x017, // reg 1 - right line volume (0 db)
        	(2<<9) | 0x0f9, // reg 2 - left headphone volume (0 db)
        	(3<<9) | 0x0f9, // reg 3 - right headphone volume (0 db)
        	(4<<9) | 0x012, // reg 4 - analog audio path (DAC on, line input)
//        	(4<<9) | 0x014, // reg 4 - analog audio path (DAC on, mic input, 0db)
//        	(4<<9) | 0x015, // reg 4 - analog audio path (DAC on, mic input, +20db)
        	(5<<9) | 0x000, // reg 5 - digital audio path (no deemphasis)
        	(6<<9) | 0x000, // reg 6 - power down (all on)
        	(7<<9) | 0x053, // reg 7 - digital audio interface (DSP mode)
        	(8<<9) | 0x001, // reg 8 - sample rate (48kHz)
        	(9<<9) | 0x001  // reg 9 - digital interface (activate)
        	// reg 9 is done automatically as part of reg 8
	};

	
	control_port->spcr = 0;          	// reset port
	control_port->pcr  = 0x00000A0A; 	// fsxm/clkx outputs, spi format
	control_port->rcr  = 0x00000000;
	control_port->xcr  = 0x00000040; 	// single 16-bit word
	control_port->srgr = 0x20100F04; 	// fs on write, 16-bits,  
	control_port->spcr = 0x00C00000; 	// start fsg 
	for(i = 0;i < 1000;i++)
		;
	control_port->spcr = 0x00C11000; 	// enable tx, clkstp for spi

	for(i = 0;i < NUM_AIC23_REGISTERS - 1;i++) {
		Write_AIC23_Reg(AIC23_data[i], control_port);
	}
	
	Init_McBSP(data_port, AIC23_Mode); // initialize data link codec
	return 48000.0; 			// default sample rate set in codec
}				 

float InitSerialComm(int CodecType)
{
	float SampleFreq = 0;
	
	if(CodecType == AIC23_16bit_McBSP01) {
 		*(unsigned volatile int *)IMH = 0x3DC03583; // assign McBSP1 to INT14/15	
		SampleFreq = Init_AIC23(McBSP0_Base, McBSP1_Base);// initialize codec using McBSP0 
		McBSP_Base_ToUse = McBSP1_Base;
	 	IER |= 0xC002;  							// enable McBSP1 Rx/Tx interrupt (14/15)  
	}	
	else if(CodecType == AIC23_16bit_McBSP12) {
 		*(unsigned volatile int *)IMH = 0x4A203583; // assign McBSP2 to INT14/15	
		SampleFreq = Init_AIC23(McBSP1_Base, McBSP2_Base);// initialize codec using McBSP1 
		McBSP_Base_ToUse = McBSP2_Base;
	 	IER |= 0xC002;  							// enable McBSP2 Rx/Tx interrupt (14/15)  
	}	
	else {
		if(C62_McBSP_IMH_Bug()) // check for IMH bug in Rev 1.X silicon 
  			*(unsigned volatile int *)IMH = 0x35803DC3; // assign McBSP1 to INT11/12 
	  	else
  			*(unsigned volatile int *)IMH = 0x3DC03583; // assign McBSP0 to INT11/12 
  		
		switch(CodecType) { // init McBSP and interrupts
		case eDAC_Audio: // Integrated DSP eDAC-Audio using AD77 codec 
		case eDSP_PCM3006: // PCM3006 I2S master
		case TI_PCM3003_16bit: // PCM3003 I2S slave 
		case eDAC_PCM3003: // Integrated DSP eDAC PCM3003 I2S master
	 		SampleFreq = SetNewFs_PCM300X(CodecType, EXTERNAL_CLOCK);
			Init_McBSP(McBSP1_Base, (CodecType==TI_PCM3003_16bit)?PCM300X_Slave_Inverted:PCM300X_Master); 
			McBSP_Base_ToUse = McBSP1_Base;
	 		IER |= 0xC002;  // enable McBSP1 Rx/Tx interrupt (14/15)  
			break;
		default: // must be onboard codec 
			Init_McBSP(McBSP0_Base, AD535_Mode);
			McBSP_Base_ToUse = McBSP0_Base;
	 	 	IER |= 0x1802;  // enable McBSP0 Rx/Tx interrupts (11/12)
	 		SampleFreq = 8000.0;
	 		break;
	 	}
	}
	return SampleFreq;
}

float SetNewFs_PCM300X(int CodecType, unsigned int div)
{
	float SampleFreq = 0;
	
	switch(CodecType) {
	case eDSP_PCM3006: // uses Timer1
		if(div == EXTERNAL_CLOCK) 
			return 48000.0;
		else {
			*(volatile unsigned int *)TIMER1_CTRL = 0;		/* stop timer */
			/* freq = 37.5Mhz / (2 * divider) */
		  	*(volatile unsigned int *)TIMER1_PRD = div;		/* set period */
			*(volatile unsigned int *)TIMER1_CTRL = 0x3D1;	/* clock mode, int clock */
			SampleFreq = ((TIMER_CLOCK/2) / 256) / div;
		}
		break;
	case TI_PCM3003_16bit: // uses Timer0 
		if(div == EXTERNAL_CLOCK) 
			return 48000.0;
		else {
			*(volatile unsigned int *)TIMER0_CTRL = 0;		/* stop timer */
			/* freq = 37.5Mhz / (2 * divider) */
		  	*(volatile unsigned int *)TIMER0_PRD = div;		/* set period */
			*(volatile unsigned int *)TIMER0_CTRL = 0x3D1;	/* clock mode, int clock */
			SampleFreq = ((TIMER_CLOCK/2) / 256) / div;
		}
		break;
	case eDAC_PCM3003: // uses Timer0
		*(unsigned volatile int *)EMIF_CE1 = CE1_32;/* EMIF CE1 control, 32-bit async */
		if(div == EXTERNAL_CLOCK) {
			*(int *)IO_PORT=0x10000000; // set fixed Fs 
			return 48000.0;
		}
		else {
			*(int *)IO_PORT=0x18000000; // set variable Fs 
			*(volatile unsigned int *)TIMER0_CTRL = 0;		/* stop timer */
			/* freq = 37.5Mhz / (2 * divider) */
		  	*(volatile unsigned int *)TIMER0_PRD = div;		/* set period */
			*(volatile unsigned int *)TIMER0_CTRL = 0x3D1;	/* clock mode, int clock */		
			SampleFreq = ((TIMER_CLOCK/2) / 256) / div;
		}
		break;
	case eDAC_Audio: // uses Timer0
		*(unsigned volatile int *)EMIF_CE2 = CE2_8;/* EMIF CE2 control, 8-bit async */
		if(div == EXTERNAL_CLOCK) {
			*(volatile int *)0xB0200000 = 0x70000000; // set fixed Fs
			*(volatile int *)0xB0200000 = 0x71000000;
			return 96000.0;
		}
		else {
			*(volatile int *)0xB0200000 = 0x30000000; // set variable Fs
			*(volatile int *)0xB0200000 = 0x31000000;
			*(volatile unsigned int *)TIMER0_CTRL = 0;		/* stop timer */
			/* freq = 37.5Mhz / (2 * divider) */
		  	*(volatile unsigned int *)TIMER0_PRD = div;		/* set period */
			*(volatile unsigned int *)TIMER0_CTRL = 0x3D1;	/* clock mode, int clock */		
			SampleFreq = ((TIMER_CLOCK/2) / 256) / div;
		}
		break;
	}
	return SampleFreq;	
}

float SetTimer0Period(unsigned int div)
{               
	*(volatile unsigned int *)TIMER0_CTRL = 0;		/* stop timer */
	/* freq = 37.5Mhz / (2 * divider) */
  	*(volatile unsigned int *)TIMER0_PRD = div;		/* set freq ~9.5Mhz */
	*(volatile unsigned int *)TIMER0_CTRL = 0x3D1;	/* clock mode, int clock */

// **************************
// need to force to 6211 code for divide and int->float
// **************************
	return (TIMER_CLOCK/2) / div;
}

// tlc320ad535 codec support                
volatile unsigned int SecCommData;
volatile int TxData = 0, RecvData = 0, SecFlag = 0;
volatile int TxFlag = PRI_COMM;  

void RequestSecCommWrite(unsigned int data)
{                      
	while(TxFlag != PRI_COMM) // wait for pending sec comm request 
		;
	SecCommData = data & 0xEFFF; // flag as write 
	TxFlag = SEC_COMM_PENDING; 
}

unsigned int RequestSecCommRead(unsigned int data)
{                      
	while(TxFlag != PRI_COMM) // wait for pending sec comm request
		;
	SecCommData = data | 0x2000; // flag as read  
	SecFlag = 4; // will be 4 irqs before sec comm recvd)
	TxFlag = SEC_COMM_PENDING; 
	while(SecFlag) // wait for sec comm received 
		;
	return RecvData; // return data last read 
}


void Init_McBSP(McBSP *port, int mode)
{   
	switch(mode) {
	case PCM300X_Master:
		port->spcr = 0;          /* reset port */
		port->pcr  = 0x00000A03; /* fsx/clkx output, fsr, clkr input */                          
		/* srg set up for clks/8, 32 bits, fsr width = 16bits */
		port->srgr = (1 << 28) + (31 << 16) + (15 << 8) + (7);
		port->rcr  = 0x000400A0; /* 1-phase, 32-bits */
		port->xcr  = 0x000400A0; 
		/* delay */
		port->spcr = 0x00400000; /* start srg */ 
		/* delay */
		port->spcr = 0x00C00000; /* start fsg */ 
		/* delay */
		port->spcr = 0x00C10001; /* enable tx/rx */
		port->dxr  = 0;
		break;
	case PCM300X_Slave:
		port->spcr = 0;          /* reset port */
		port->pcr  = 0x00000003; /* fsx/clkx/fsr/clkr input */                          
		port->srgr = 0;  		 /* srgr disabled */
		port->rcr  = 0x000400A0; /* 1-phase, 32-bits */
		port->xcr  = 0x000400A0; 
		port->spcr = 0x00010001; /* enable tx/rx */
		port->dxr  = 0;
		break;	
	case PCM300X_Slave_Inverted:
		port->spcr = 0;          /* reset port */
		port->pcr  = 0x00000000; /* fs/clk input */                          
		port->srgr = 0;  		 /* srgr disabled */
		port->rcr  = 0x000400A0; /* 1-phase, 32-bits */
		port->xcr  = 0x000400A0; 
		port->spcr = 0x00010001; /* enable tx/rx */
		port->dxr  = 0;
		break;
	case AIC23_Mode: // using AIC23 codec in DSP mode
		port->spcr = 0;          /* reset port */
		port->pcr  = 0x0000000f; /* fsx/clkx/fsr/clkr input, fs active low */                          
		port->srgr = 0;  		 /* srgr disabled */
		port->rcr  = 0x000400A0; /* 1-phase, 32-bits */
		port->xcr  = 0x000400A0; 
		port->spcr = 0x00010001; /* enable tx/rx */
		port->dxr  = 0;
		break;	
	case AD535_Mode: 
		port->spcr = 0;
		port->pcr  = 0;
		port->rcr  = 0x10040;
		port->xcr  = 0x10040;
		port->spcr = 0x12001;  
		port->dxr  = 0;
		break;
	}
}

int EMIF_Init()
{
  unsigned int pwb_assy_rev;
  int mem_size;

  *(unsigned volatile int *)EMIF_CE1 = CE1_32;// EMIF CE1 control, 32bit async
  pwb_assy_rev = (((*(unsigned volatile int *)IO_PORT)>>29)&0x7);   
                                                                              
/*************************************************************************
* Standard 6211 DSK includes 2 MT48LC1M16A1-7 devices =>4MB SDRAM        *
*   16Mb (16-bit x 2 banks x 512K) parts = 2MB / part                    *
*   EMIF_SDCTRL=0x07227000                                               *
*   EMIF_SDEXT=0x54529                         Board Rev = 1             *
*------------------------------------------------------------------------*
* Standard 6711 DSK includes 2 MT48LC4M16A2-8 devices =>16MB SDRAM       *
*   64Mb (16-bit x 4 banks x 1M)   parts = 8MB / part                    *
*   EMIF_SDCTRL=0x57116000                                               *
*   EMIF_SDEXT=0x54529 (Hitachi 0x54509)       Board Rev = 2             *
*------------------------------------------------------------------------*
* Other 6711 DSK configurations are as follows:                          *
*   128Mb (16-bit x 4 banks x 2M)  parts = 16MB / part (=>32MB SDRAM)    *
*   EMIF_SDCTRL=0x53116000                                               *
*   EMIF_SDEXT=0x54529 (Hitachi 0x54509)       Board Rev = 3             *
*   ---------------------------------------------------------------------*
*   256Mb (16-bit x 4 banks x 4M)  parts = 32MB / part (=>64MB SDRAM)    *
*   EMIF_SDCTRL=0x63116000                                               *
*   EMIF_SDEXT=0x54529                         Board Rev = 4             *
*************************************************************************/

  *(unsigned volatile int *)EMIF_GCR = 0x3300;    /* EMIF global control      */
  *(unsigned volatile int *)EMIF_CE1 = CE1_32;    /* EMIF CE1 control, 32bit  */
  *(unsigned volatile int *)EMIF_SDCTRL = 0x07227000; /* EMIF SDRAM control   */ 
  *(unsigned volatile int *)EMIF_CE0 = 0x30;      /* EMIF CE0 control         */
  *(unsigned volatile int *)EMIF_SDRP = 0x61a;    /* EMIF SDRM refresh period */
  *(unsigned volatile int *)EMIF_SDEXT= 0x54529;  /* EMIF SDRM extension      */

  /*-----------------------------------------------------------------------*/
  /* Modify SDRAM parameter value(s) according to board revision           */
  /*-----------------------------------------------------------------------*/
  switch(pwb_assy_rev) {
  case 1:
    *(unsigned volatile int *)EMIF_SDCTRL = 0x07227000; 
    mem_size = 0x0400000;
    break;
  case 2:
    *(unsigned volatile int *)EMIF_SDCTRL = 0x57116000;
    mem_size = 0x1000000;
    break;
  case 3:
    *(unsigned volatile int *)EMIF_SDCTRL = 0x53116000;
    mem_size = 0x2000000;
    break;
  case 4:
    *(unsigned volatile int *)EMIF_SDCTRL = 0x63116000;
    mem_size = 0x4000000;
    break;
  default:
    *(unsigned volatile int *)EMIF_SDCTRL = 0x07126000;
    mem_size = 0x0400000;
    break;
  }
  return mem_size;
} 

int C62_McBSP_IMH_Bug() /* check for IMH bug in Rev 1.X silicon */
{   					/* McBSP0 and McBSP1 switched in interrupt mux */           
	McBSP *port0  = McBSP0_Base, *port1  = McBSP1_Base; 
	volatile int i;
	
  	IER=1;                  /* disable all interrupts except NMI */
  	*(unsigned volatile int *)IMH = 0x08203DC3; /* assign McBSP1 Tx/Rx to INT11/12 */
 	ICR=0xffff;             /* clear all pending interrupts      */
 	port1->spcr = 0; 		/* reset McBSP1 */
 	port0->spcr = 0; 		/* reset McBSP0 */
 	ICR=0xffff;             /* clear all pending interrupts      */
	port0->spcr = 0x312031; /* McBSP0 irq on sync error */
	port0->spcr = 0x392039; /* cause McBSP0 rx/tx sync error */

	i = 20;	                /* delay for irq recognition */
	while(i--)
		;

	if((IFR & 0x1800) == 0x1800) /* IFR set by McBSP1, so is Rev 1.X silicon */
		return 1;
	return 0;
}
